@@ -20,9 +20,10 @@ from datetime import datetime
 from hashlib import blake2b
 from typing import Union
 
+from scalecodec.constants import TYPE_DECOMP_MAX_RECURSIVE
 from scalecodec.utils.ss58 import ss58_decode_account_index, ss58_decode, ss58_encode, is_valid_ss58_address
 
-from scalecodec.base import ScaleType, ScaleBytes
+from scalecodec.base import ScaleType, ScaleBytes, ScalePrimitive
 from scalecodec.exceptions import InvalidScaleTypeValueException, MetadataCallFunctionNotFound
 from scalecodec.utils.math import trailing_zeros, next_power_of_two
 
@@ -90,6 +91,14 @@ class Compact(ScaleType):
             else:
                 raise ValueError('{} out of range'.format(value))
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+        if cls.sub_type is None:
+            return cls.__name__
+
+        scale_obj = cls.runtime_config.create_scale_object(cls.sub_type)
+        return scale_obj.generate_type_decomposition(_recursion_level=_recursion_level + 1)
+
 
 class CompactU32(Compact):
     """
@@ -149,6 +158,16 @@ class Option(ScaleType):
     def process_scale_info_definition(cls, scale_info_definition: 'GenericRegistryType', prefix: str):
         cls.sub_type = f"{prefix}::{scale_info_definition.value['params'][0]['type']}"
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+
+        if cls.sub_type is None:
+            raise ValueError("'sub_type' is not set")
+
+        sub_type_obj = cls.runtime_config.create_scale_object(cls.sub_type)
+        return None, sub_type_obj.generate_type_decomposition(_recursion_level=_recursion_level + 1)
+
+
 
 class Bytes(ScaleType):
 
@@ -268,7 +287,7 @@ class RawBytes(ScaleType):
         return ScaleBytes(bytes.fromhex(value[2:]))
 
 
-class U8(ScaleType):
+class U8(ScalePrimitive):
 
     def process(self):
         return self.get_next_u8()
@@ -281,7 +300,7 @@ class U8(ScaleType):
             raise ValueError('{} out of range for u8'.format(value))
 
 
-class U16(ScaleType):
+class U16(ScalePrimitive):
 
     def process(self):
         return int.from_bytes(self.get_next_bytes(2), byteorder='little')
@@ -294,7 +313,7 @@ class U16(ScaleType):
             raise ValueError('{} out of range for u16'.format(value))
 
 
-class U32(ScaleType):
+class U32(ScalePrimitive):
 
     def process(self):
         return int.from_bytes(self.get_next_bytes(4), byteorder='little')
@@ -307,7 +326,7 @@ class U32(ScaleType):
             raise ValueError('{} out of range for u32'.format(value))
 
 
-class U64(ScaleType):
+class U64(ScalePrimitive):
 
     def process(self):
         return int(int.from_bytes(self.get_next_bytes(8), byteorder='little'))
@@ -320,7 +339,7 @@ class U64(ScaleType):
             raise ValueError('{} out of range for u64'.format(value))
 
 
-class U128(ScaleType):
+class U128(ScalePrimitive):
 
     def process(self):
         return int(int.from_bytes(self.get_next_bytes(16), byteorder='little'))
@@ -333,7 +352,7 @@ class U128(ScaleType):
             raise ValueError('{} out of range for u128'.format(value))
 
 
-class U256(ScaleType):
+class U256(ScalePrimitive):
 
     def process(self):
         return int(int.from_bytes(self.get_next_bytes(32), byteorder='little'))
@@ -346,7 +365,7 @@ class U256(ScaleType):
             raise ValueError('{} out of range for u256'.format(value))
 
 
-class I8(ScaleType):
+class I8(ScalePrimitive):
 
     def process(self):
         return int.from_bytes(self.get_next_bytes(1), byteorder='little', signed=True)
@@ -359,7 +378,7 @@ class I8(ScaleType):
             raise ValueError('{} out of range for i8'.format(value))
 
 
-class I16(ScaleType):
+class I16(ScalePrimitive):
 
     def process(self):
         return int.from_bytes(self.get_next_bytes(2), byteorder='little', signed=True)
@@ -372,7 +391,7 @@ class I16(ScaleType):
             raise ValueError('{} out of range for i16'.format(value))
 
 
-class I32(ScaleType):
+class I32(ScalePrimitive):
 
     def process(self):
         return int.from_bytes(self.get_next_bytes(4), byteorder='little', signed=True)
@@ -385,7 +404,7 @@ class I32(ScaleType):
             raise ValueError('{} out of range for i32'.format(value))
 
 
-class I64(ScaleType):
+class I64(ScalePrimitive):
 
     def process(self):
         return int.from_bytes(self.get_next_bytes(8), byteorder='little', signed=True)
@@ -398,7 +417,7 @@ class I64(ScaleType):
             raise ValueError('{} out of range for i64'.format(value))
 
 
-class I128(ScaleType):
+class I128(ScalePrimitive):
 
     def process(self):
         return int.from_bytes(self.get_next_bytes(16), byteorder='little', signed=True)
@@ -411,7 +430,7 @@ class I128(ScaleType):
             raise ValueError('{} out of range for i128'.format(value))
 
 
-class I256(ScaleType):
+class I256(ScalePrimitive):
 
     def process(self):
         return int.from_bytes(self.get_next_bytes(32), byteorder='little', signed=True)
@@ -424,7 +443,7 @@ class I256(ScaleType):
             raise ValueError('{} out of range for i256'.format(value))
 
 
-class F32(ScaleType):
+class F32(ScalePrimitive):
 
     def process(self):
         return struct.unpack('f', self.get_next_bytes(4))[0]
@@ -436,7 +455,7 @@ class F32(ScaleType):
         return ScaleBytes(struct.pack('f', value))
 
 
-class F64(ScaleType):
+class F64(ScalePrimitive):
 
     def process(self):
         return struct.unpack('d', self.get_next_bytes(8))[0]
@@ -448,7 +467,7 @@ class F64(ScaleType):
         return ScaleBytes(struct.pack('d', value))
 
 
-class H160(ScaleType):
+class H160(ScalePrimitive):
 
     def process(self):
         return '0x{}'.format(self.get_next_bytes(20).hex())
@@ -459,7 +478,7 @@ class H160(ScaleType):
         return ScaleBytes(value)
 
 
-class H256(ScaleType):
+class H256(ScalePrimitive):
 
     def process(self):
         return '0x{}'.format(self.get_next_bytes(32).hex())
@@ -470,7 +489,7 @@ class H256(ScaleType):
         return ScaleBytes(value)
 
 
-class H512(ScaleType):
+class H512(ScalePrimitive):
 
     def process(self):
         return '0x{}'.format(self.get_next_bytes(64).hex())
@@ -511,11 +530,15 @@ class Struct(ScaleType):
 
         return result
 
-    def process_encode(self, value):
+    def process_encode(self, value: Union[dict, tuple]) -> ScaleBytes:
         data = ScaleBytes(bytearray())
 
         self.value_object = {}
 
+        if type(value) is tuple:
+            # Convert tuple to dict
+            value = {key: value[idx] for idx, (key, _) in enumerate(self.type_mapping)}
+
         for key, data_type in self.type_mapping:
             if key not in value:
                 raise ValueError('Element "{}" of struct is missing in given value'.format(key))
@@ -528,6 +551,20 @@ class Struct(ScaleType):
 
         return data
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+
+        if _recursion_level > TYPE_DECOMP_MAX_RECURSIVE or cls.type_mapping is None:
+            raise ValueError("'type_mapping' is not set")
+
+        result = {}
+        for key, data_type in cls.type_mapping:
+            if data_type is not None:
+                scale_obj = cls.runtime_config.create_scale_object(data_type)
+                data_type = scale_obj.generate_type_decomposition(_recursion_level=_recursion_level + 1)
+            result[key] = data_type
+        return result
+
 
 class Tuple(ScaleType):
     def __init__(self, data=None, type_mapping=None, **kwargs):
@@ -578,6 +615,21 @@ class Tuple(ScaleType):
 
         return data
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+        result = ()
+        if cls.type_mapping:
+            for member_type in cls.type_mapping:
+                if member_type is not None:
+                    scale_obj = cls.runtime_config.create_scale_object(member_type)
+                    member_type = scale_obj.generate_type_decomposition(_recursion_level=_recursion_level + 1)
+
+                    if len(cls.type_mapping) == 1:
+                        return member_type
+
+                result += (member_type,)
+        return result
+
 
 class Set(ScaleType):
     value_list = []
@@ -614,6 +666,10 @@ class Set(ScaleType):
 
         return u64_obj.encode(result)
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+        return tuple(cls.value_list)
+
 
 class Era(ScaleType):
     """
@@ -718,7 +774,7 @@ class Era(ScaleType):
         return
 
 
-class Bool(ScaleType):
+class Bool(ScalePrimitive):
 
     def process(self):
         return self.get_next_bool()
@@ -795,6 +851,10 @@ class GenericAccountId(H256):
     def process_scale_info_definition(cls, scale_info_definition: 'GenericRegistryType', prefix: str):
         return
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+        return 'AccountId'
+
 
 class GenericEthereumAccountId(H160):
 
@@ -885,6 +945,21 @@ class Vec(ScaleType):
     def __len__(self):
         return len(self.value_object)
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+
+        if cls.sub_type is None:
+            raise ValueError("'sub_type' is not set")
+
+        sub_obj = cls.runtime_config.create_scale_object(cls.sub_type)
+        sub_type_decomp = sub_obj.generate_type_decomposition(_recursion_level=_recursion_level + 1)
+
+        if sub_type_decomp == 'u8':
+            # Translate Vec<u8> to Bytes
+            return 'Bytes'
+        else:
+            return [sub_obj.generate_type_decomposition(_recursion_level=_recursion_level + 1)]
+
 
 class BoundedVec(Vec):
     def __init__(self, data=None, **kwargs):
@@ -938,6 +1013,10 @@ class BitVec(ScaleType):
 
         return data + value.to_bytes(length=byte_length, byteorder='little')
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+        return 'BitVec'
+
 
 class GenericAddress(ScaleType):
 
@@ -1000,6 +1079,10 @@ class GenericAddress(ScaleType):
         else:
             return self.value
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+        return cls.__name__.lower()
+
 
 class AccountIdAddress(GenericAddress):
 
@@ -1128,6 +1211,30 @@ class Enum(ScaleType):
             else:
                 return self.value_list[self.index]
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+
+        if _recursion_level > TYPE_DECOMP_MAX_RECURSIVE:
+            return cls.__name__
+
+        if cls.type_mapping:
+
+            # Check if type_mapping can be converted to simple value_list
+            if all([t == 'Null' for n, t in cls.type_mapping]):
+                return tuple([n for n, t in cls.type_mapping if n is not None])
+
+            result = {}
+            for key, data_type in cls.type_mapping:
+                if data_type == 'Null':
+                    data_type = None
+                if data_type is not None:
+                    scale_obj = cls.runtime_config.create_scale_object(data_type)
+                    data_type = scale_obj.generate_type_decomposition(_recursion_level=_recursion_level + 1)
+                result[key] = data_type
+            return result
+        else:
+            return tuple(cls.value_list)
+
 
 class Data(Enum):
     type_mapping = [
@@ -1300,6 +1407,13 @@ class GenericVote(U8):
 
         return super().process_encode(value)
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+        return {
+            'aye': 'bool',
+            'conviction': cls.runtime_config.create_scale_object('Conviction').generate_type_decomposition()
+        }
+
 
 class GenericCall(ScaleType):
 
@@ -1335,18 +1449,24 @@ class GenericCall(ScaleType):
             if len(self.call_args) > 0:
 
                 # Check args format
-                if type(call_obj[1].value) is not tuple:
-                    call_args_values = (call_obj[1],)
-                else:
-                    call_args_values = call_obj[1]
+                if type(call_obj[1].value) is dict:
 
-                for idx, call_arg in enumerate(self.call_args):
-                    call_args.append({
-                        'name': call_arg.value['name'],
-                        'type': self.convert_type(call_arg.value['typeName']),
-                        'value': call_args_values[idx].value
-                    })
-                    self.call_args[idx].value_object['value'] = call_args_values[idx]
+                    for idx, call_arg in enumerate(self.call_args):
+                        call_args.append({
+                            'name': call_arg.value['name'],
+                            'type': self.convert_type(call_arg.value['typeName']),
+                            'value': call_obj[1].value[call_arg.value['name']]
+                        })
+                        self.call_args[idx].value_object['value'] = call_obj[1][call_arg.value['name']]
+                else:
+                    # Backwards compatibility
+                    for idx, call_arg in enumerate(self.call_args):
+                        call_args.append({
+                            'name': call_arg.value['name'],
+                            'type': self.convert_type(call_arg.value['typeName']),
+                            'value': call_obj[1][idx].value
+                        })
+                        self.call_args[idx].value_object['value'] = call_obj[1][idx]
 
             self.value_object = {
                 'call_index': f'0x{self.call_index}',
@@ -1451,6 +1571,10 @@ class GenericCall(ScaleType):
             if len(self.call_args) > 0:
                 self.value_object['call_args'] = {}
 
+                # convert alternative list format of call_args
+                if type(value['call_args']) is list:
+                    value['call_args'] = {ca['name']: ca['value'] for ca in value['call_args']}
+
                 for arg in self.call_args:
                     if arg.value['name'] not in value['call_args']:
                         raise ValueError('Parameter \'{}\' not specified'.format(arg.value['name']))
@@ -1514,6 +1638,10 @@ class GenericCall(ScaleType):
 
             return data
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+        return 'Call'
+
 
 class GenericContractExecResult(Enum):
     def __init__(self, data=None, contract_result_scale_type=None, **kwargs):
@@ -1605,6 +1733,18 @@ class WrapperKeepOpaque(Struct):
             self.value_object = bytes_obj.value_object
             return f'0x{bytes_obj.value_object.hex()}'
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+
+        if cls.type_mapping is None:
+            raise ValueError("'type_mapping' is not set")
+
+        # Return decomposition of wrapped type
+        wrapped_obj = cls.runtime_config.create_scale_object(
+            type_string=cls.type_mapping[1]
+        )
+        return wrapped_obj.generate_type_decomposition(_recursion_level=_recursion_level + 1)
+
 
 class MultiAccountId(GenericAccountId):
 
@@ -1668,6 +1808,9 @@ class FixedLengthArray(ScaleType):
             if type(value) is str and value[0:2] == '0x':
                 value = bytes.fromhex(value[2:])
 
+            if type(value) is list:
+                value = bytes(value)
+
             if type(value) is not bytes:
                 raise ValueError('Value should a hex-string (0x..) or bytes')
 
@@ -1689,6 +1832,14 @@ class FixedLengthArray(ScaleType):
 
             return data
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+        if cls.sub_type is None:
+            raise ValueError("'sub_type' is not set")
+
+        sub_cls = cls.runtime_config.get_decoder_class(cls.sub_type)
+        return f'[{sub_cls.generate_type_decomposition(_recursion_level=_recursion_level + 1)}; {cls.element_count}]'
+
 
 class GenericMultiAddress(Enum):
     type_mapping = [
@@ -2052,35 +2203,6 @@ class GenericRegistryType(Struct):
 
         return super().process_encode(value)
 
-    def retrieve_type_decomposition(self):
-        if 'variant' in self.value['def']:
-            for variant in self.value['def']['variant']['variants']:
-                for field in variant['fields']:
-                    field_obj = self.runtime_config.create_scale_object(f"scale_info::{field['type']}")
-                    variant['value'] = field_obj.scale_info_type.retrieve_type_decomposition()
-
-        elif 'composite' in self.value['def']:
-            for field in self.value['def']['composite']['fields']:
-                field_obj = self.runtime_config.create_scale_object(f"scale_info::{field['type']}")
-                field['value'] = field_obj.scale_info_type.retrieve_type_decomposition()
-
-        elif 'array' in self.value['def']:
-            type_def = self.value['def']['array']
-            array_element_obj = self.runtime_config.create_scale_object(f"scale_info::{type_def['type']}")
-            type_def['value'] = array_element_obj.scale_info_type.retrieve_type_decomposition()
-
-        elif 'sequence' in self.value['def']:
-            type_def = self.value['def']['sequence']
-            array_element_obj = self.runtime_config.create_scale_object(f"scale_info::{type_def['type']}")
-            type_def['value'] = array_element_obj.scale_info_type.retrieve_type_decomposition()
-
-        elif 'tuple' in self.value['def']:
-            for idx, type_def in self.value['def']['tuple']:
-                element_obj = self.runtime_config.create_scale_object(f"scale_info::{type_def}")
-                self.value['def']['tuple'][idx] = element_obj.scale_info_type.retrieve_type_decomposition()
-
-        return self.value['def']
-
 
 class GenericField(Struct):
 
@@ -2131,7 +2253,7 @@ class GenericVariant(Struct):
         for arg in self.args:
             param_type_obj = self.runtime_config.create_scale_object(arg.type)
 
-            param_info[arg.name] = param_type_obj.scale_info_type.retrieve_type_decomposition()
+            param_info[arg.name] = param_type_obj.generate_type_decomposition()
 
         return param_info
 
@@ -2453,10 +2575,35 @@ class ScaleInfoStorageEntryMetadata(GenericStorageEntryMetadata):
             raise NotImplementedError()
 
     def get_param_info(self) -> list:
+        """
+        Return a type decomposition how to format parameters for current storage function
+
+        Returns
+        -------
+        list
+        """
         param_info = []
         for param_type_string in self.get_params_type_string():
             scale_type = self.runtime_config.create_scale_object(param_type_string)
-            param_info.append(scale_type.scale_info_type.retrieve_type_decomposition())
+            param_info.append(scale_type.generate_type_decomposition())
+
+        return param_info
+
+
+class GenericRuntimeCallDefinition(Struct):
+
+    def get_param_info(self) -> list:
+        """
+        Return a type decomposition how to format parameters for current storage function
+
+        Returns
+        -------
+        list
+        """
+        param_info = []
+        for param in self.value['params']:
+            scale_type = self.runtime_config.create_scale_object(param['type'])
+            param_info.append(scale_type.generate_type_decomposition())
 
         return param_info
 
@@ -2623,6 +2770,10 @@ class GenericExtrinsic(ScaleType):
 
         return data
 
+    @classmethod
+    def generate_type_decomposition(cls, _recursion_level: int = 0):
+        return 'Extrinsic'
+
 
 class Extrinsic(GenericExtrinsic):
     pass
